searchData={"items":[{"type":"task","title":"mix execute","doc":"Compiles and runs a file using the `Signo` compiler.\n\n    $ mix execute hello.sg\n    hello, world!","ref":"Mix.Tasks.Execute.html"},{"type":"task","title":"mix repl","doc":"Starts a REPL (read-evaluate-print loop) session.\n\n    $ mix repl\n    Erlang/OTP 26 [erts-14.2.1] [source] [64-bit] [smp:11:11] [ds:11:11:10] [async-threads:1] [jit]\n\n    Interactive Signo v0.1.0 (Elixir/1.16.2)\n    sig(1)> (print \"hello world\")\n    hello world\n    #ok\n    sig(2)>\n\nTurns out Elixir and Erlang make something as simple as a readline *a fucking nightmare*\nto implement. So if you want a decent typing experience, consider installing\n[rlwrap](https://github.com/hanslub42/rlwrap) and running the REPL like this:\n\n    rlwrap mix repl","ref":"Mix.Tasks.Repl.html"},{"type":"module","title":"Signo","doc":"Signo is a beautifully elegant Lisp-like language, written in Elixir.\n\nThis module contains the public (Elixir) API for evaluating Signo programs. Check\nout the \"Pages\" tab in the left-hand corner for details on installing and running\nthe interpreter, along with a detailed guide to the Signo language.\n\nYou'll mainly be using `eval_file!/1` and `eval_source!/1`, or a CLI abstraction\nof them, for compiling and interpreting your Signo source code. However, for more\nadvanced usecases, we also expose individual compiler steps, such as `lex!/1`,\n`parse!/1`, `evaluate!/1`.\n\nSee \"mix tasks\" in the left-hand corner for details on command-line usage.","ref":"Signo.html"},{"type":"function","title":"Signo.eval_file!/1","doc":"Compiles and evaluates a Signo source file.","ref":"Signo.html#eval_file!/1"},{"type":"function","title":"Examples - Signo.eval_file!/1","doc":"iex> Signo.eval_file!(\"./main.sg\")\n    hello, world!\n    {%AST.String{value: \"hello world\"}, %Signo.Env{...}}","ref":"Signo.html#eval_file!/1-examples"},{"type":"function","title":"Signo.eval_source!/1","doc":"Compiles and evaluates a string of Signo source code.","ref":"Signo.html#eval_source!/1"},{"type":"function","title":"Examples - Signo.eval_source!/1","doc":"iex> Signo.eval_source!(\"(print 69)\")\n    69\n    {%AST.Atom{value: :ok}, %Signo.Env{...}}","ref":"Signo.html#eval_source!/1-examples"},{"type":"function","title":"Signo.evaluate!/1","doc":"Evaluates a `Signo.AST` into a `Signo.Env` containing final\nglobal scope, and executes any side-effects.\n\nSee \"Exceptions\" for potential exceptions that can be raised.","ref":"Signo.html#evaluate!/1"},{"type":"function","title":"Signo.evaluate!/2","doc":"Same as `evaluate!/1`, but operates on an existing scope instead of\nintializing a new one.\n\nPrimarily used to facilitate REPL-like programs, but can be applied\nin other contexts as well.","ref":"Signo.html#evaluate!/2"},{"type":"function","title":"Signo.lex!/1","doc":"Lexes a string containing valid Signo source code into\na list of `Signo.Token`s.\n\nRaises `Signo.LexError` when encountering unknown characters.\nMultiple lines are supported.","ref":"Signo.html#lex!/1"},{"type":"function","title":"Signo.parse!/1","doc":"Parses a list of `Signo.Token`s into a executable AST.\n\nRaises `Signo.ParseError` when encountering unexpected tokens.","ref":"Signo.html#parse!/1"},{"type":"function","title":"Signo.repl/0","doc":"Starts a REPL (read-evaluate-print loop) session.","ref":"Signo.html#repl/0"},{"type":"function","title":"Examples - Signo.repl/0","doc":"Erlang/OTP 26 [erts-14.2.1] [source] [64-bit] [smp:11:11] [ds:11:11:10] [async-threads:1] [jit]\n\n    Interactive Signo v0.1.0 (Elixir/1.16.2)\n    sig(1)> (print \"hello world\")\n    hello world\n    sig(2)>","ref":"Signo.html#repl/0-examples"},{"type":"function","title":"Signo.sigil_l/2","doc":"Shorthand for `lex!/1` and `parse!/1`.\n\nGiven a string of Signo source code, returns a valid\nAST that can be evaluated using `evaluate!/1`.","ref":"Signo.html#sigil_l/2"},{"type":"function","title":"Examples - Signo.sigil_l/2","doc":"iex> evaluate!(~l\"(print 10)\")\n    {%Signo.AST.Atom{value: :ok}, %Signo.AST.Env{}}","ref":"Signo.html#sigil_l/2-examples"},{"type":"function","title":"Signo.version/0","doc":"Returns the Signo version string.","ref":"Signo.html#version/0"},{"type":"module","title":"Signo.Env","doc":"Struct holding all variable and function definitions in\na certain scope.\n\nScopes can be nested. Whenever a variable is being looked up,\nthe lookup will walk the scope-chain up until it finds a definition.","ref":"Signo.Env.html"},{"type":"function","title":"Signo.Env.assign/4","doc":"","ref":"Signo.Env.html#assign/4"},{"type":"type","title":"Signo.Env.definitions/0","doc":"","ref":"Signo.Env.html#t:definitions/0"},{"type":"function","title":"Signo.Env.import/3","doc":"","ref":"Signo.Env.html#import/3"},{"type":"function","title":"Signo.Env.lookup!/3","doc":"","ref":"Signo.Env.html#lookup!/3"},{"type":"function","title":"Signo.Env.new/2","doc":"","ref":"Signo.Env.html#new/2"},{"type":"type","title":"Signo.Env.scope/0","doc":"","ref":"Signo.Env.html#t:scope/0"},{"type":"type","title":"Signo.Env.t/0","doc":"","ref":"Signo.Env.html#t:t/0"},{"type":"module","title":"Signo.Position","doc":"Records the position of a character or token in the original source code.","ref":"Signo.Position.html"},{"type":"function","title":"Signo.Position.increment/2","doc":"Updates the recorded pos given either a single grapheme,\nor a collected list of graphemes.","ref":"Signo.Position.html#increment/2"},{"type":"function","title":"Signo.Position.new/1","doc":"","ref":"Signo.Position.html#new/1"},{"type":"function","title":"Signo.Position.new/2","doc":"","ref":"Signo.Position.html#new/2"},{"type":"type","title":"Signo.Position.path/0","doc":"","ref":"Signo.Position.html#t:path/0"},{"type":"type","title":"Signo.Position.t/0","doc":"","ref":"Signo.Position.html#t:t/0"},{"type":"module","title":"Signo.Token","doc":"A token.","ref":"Signo.Token.html"},{"type":"type","title":"Signo.Token.literal/0","doc":"The value of the literal as an elixir `t:term/0`.\nExample: `30_000`.","ref":"Signo.Token.html#t:literal/0"},{"type":"function","title":"Signo.Token.new/3","doc":"","ref":"Signo.Token.html#new/3"},{"type":"type","title":"Signo.Token.t/0","doc":"A token.\n\nHas the following fields:\n\n- `type`: the type of token, see `t:type/0`.\n- `lexeme`: the string as found in the source code.\n- `pos`: the `Signo.Position` where the source string was found.","ref":"Signo.Token.html#t:t/0"},{"type":"type","title":"Signo.Token.type/0","doc":"","ref":"Signo.Token.html#t:type/0"},{"type":"module","title":"Signo.SpecialForms","doc":"Macro definitions for core language features.","ref":"Signo.SpecialForms.html"},{"type":"function","title":"Signo.SpecialForms._def/3","doc":"Creates a callable function and assigns in in scope.\n\n    sig> (def add (a b) (+ a b))\n     (a b -> ...)\n    sig> (add 1 2)\n    3\n\nThis is syntatic sugar for:\n\n    sig> (let add (lambda (a b) (+ a b)))\n     (a b -> ...)","ref":"Signo.SpecialForms.html#_def/3"},{"type":"function","title":"Signo.SpecialForms._do/3","doc":"Evaluates all expression in a nested scope and then\nreturns the value of the last expression.\n\n    sig> (do (let x 10)\n             (print 10))\n    10\n    #ok\n    sig> (print x)\n    [ReferenceError] x is undefined at nofile:2:8","ref":"Signo.SpecialForms.html#_do/3"},{"type":"function","title":"Signo.SpecialForms._if/3","doc":"An execution branch based a on the `condition`.\n\nIf truthy, `then` gets evaluated. If falsy, `otherwise` gets \nevaluated. If `otherwise` is not passed, and `condition` evaluated \nto a falsy value, `()` gets returned.\n\n    sig> (if (== 2 (+ 1 1)) (print \"math works\") (print \"universe is broken\"))\n    math works\n    #ok\n\n    sig> (if (== 3 (+ 1 1)) (print \"universe still broken\"))\n    ()","ref":"Signo.SpecialForms.html#_if/3"},{"type":"function","title":"Signo.SpecialForms._import/3","doc":"Similar to `include/3`, but where `include/3` exposes and mutates \nglobal scope, `import/3` executes the file in an isolated container \nand imports the resulting `Signo.Env` into global scope under a namespace.\n\n      sig> (import \"math.sg\" as math)\n      sig> (math:fact 5)\n      120\n\nIf unspecified, the namespace is derived from the given path.\n\n      sig> (import \"math.sg\")\n      sig> math:pi\n      3.14159","ref":"Signo.SpecialForms.html#_import/3"},{"type":"function","title":"Signo.SpecialForms.eval/3","doc":"Evaluates a (quoted) expression.\n\n    sig> (eval '(print 10))\n    10\n    #ok","ref":"Signo.SpecialForms.html#eval/3"},{"type":"function","title":"Signo.SpecialForms.include/3","doc":"Compiles and executes another file at runtime.\n\n    # example.sg\n    (print \"hello from example.sg!\")\n\n    sig> (include \"example.sg\")\n    hello from example.sg!\n    #ok","ref":"Signo.SpecialForms.html#include/3"},{"type":"function","title":"Signo.SpecialForms.lambda/3","doc":"Creates an (anonymous) callable function.\n\n    sig> (lambda (a b) (+ a b))\n     (a b -> ...)\n    sig> ((lambda (a b) (+ a b)) 1 2)\n    3\n\nIf the function takes only a single argument, the\nparentheses around the argument list can be omitted:\n\n    sig> (lambda n (* 2 n))\n     (n -> ...)","ref":"Signo.SpecialForms.html#lambda/3"},{"type":"function","title":"Signo.SpecialForms.let/3","doc":"Assigns a symbol to a value for current scope.\n\n    sig> (let x 10)\n    10\n    sig> (* 2 x)\n    20","ref":"Signo.SpecialForms.html#let/3"},{"type":"module","title":"Signo.StdLib","doc":"Standard Library for the Signo Programming Language.\n\n> #### A note on function names {: .neutral}\n>\n> Due to requirements for Elixir function names and conflicts with\n> existing Elixir keywords and/or operators, the function names in\n> this module are often different than the function names in Signo.\n>\n> All functions have a small example along them, which contains\n> the Signo names.","ref":"Signo.StdLib.html"},{"type":"function","title":"Signo.StdLib._and/1","doc":"Boolean `and` operator.\n\nReceives two values (not limited to booleans), and returns `#true` if both\nare truthy. Does NOT short-circuit!\n\n    sig> (and 10 #true)\n    #true\n    sig> (and #false ())\n    #false","ref":"Signo.StdLib.html#_and/1"},{"type":"function","title":"Signo.StdLib._nor/1","doc":"Boolean `nand` operator.\n\nReceives two values (not limited to booleans), and returns `#true` if\nboth are falsy. Does NOT short-circuit!\n\n    sig> (nor () #true)\n    #false\n    sig> (nor #false ())\n    #true","ref":"Signo.StdLib.html#_nor/1"},{"type":"function","title":"Signo.StdLib._not/1","doc":"Boolean `not` operator.\n\nReceives any value (not limited to booleans) and returns `#true` for falsy values,\nand `#false` for truthy ones.\n\n    sig> (not 10)\n    #false\n    sig> (not ())\n    #true","ref":"Signo.StdLib.html#_not/1"},{"type":"function","title":"Signo.StdLib._or/1","doc":"Boolean `or` operator.\n\nReceives two values (not limited to booleans), and returns `#true` if\none of them is truthy. Does NOT short-circuit!\n\n    sig> (or () #true)\n    #true\n    sig> (and #false ())\n    #false","ref":"Signo.StdLib.html#_or/1"},{"type":"function","title":"Signo.StdLib._xor/1","doc":"Boolean `xor` operator.\n\nReceives two values (not limited to booleans), and returns `#true` if\none of them is truthy, but returns `#false` if both are truthy.\n\n    sig> (xor () #true)\n    #true\n    sig> (xor #false ())\n    #false\n    sig> (xor #true 10)\n    #false","ref":"Signo.StdLib.html#_xor/1"},{"type":"function","title":"Signo.StdLib.abs/1","doc":"Arithmetical absolute value of `a`.\n\n    sig> (abs -3)\n    3\n    sig> (abs 6)\n    6","ref":"Signo.StdLib.html#abs/1"},{"type":"function","title":"Signo.StdLib.acos/1","doc":"Inverse cosine of `x` in radians.\n\n    sig> (acos -1)\n    3.14159...","ref":"Signo.StdLib.html#acos/1"},{"type":"function","title":"Signo.StdLib.add/1","doc":"Adds two numbers.\n\n    sig> (+ 2 3)\n    5","ref":"Signo.StdLib.html#add/1"},{"type":"function","title":"Signo.StdLib.asin/1","doc":"Inverse sine of `x` in radians.\n\n    sig> (asin 0)\n    3.14159...","ref":"Signo.StdLib.html#asin/1"},{"type":"function","title":"Signo.StdLib.atan/1","doc":"Inverse tangent of `x` in radians.\n\n    sig> (atan 0)\n    0","ref":"Signo.StdLib.html#atan/1"},{"type":"function","title":"Signo.StdLib.capitalize/1","doc":"Converts the first character in the given string to uppercase and the remainder to lowercase.\n\n    sig> (capitalize \"olá\")\n    \"Olá\"","ref":"Signo.StdLib.html#capitalize/1"},{"type":"function","title":"Signo.StdLib.clear/1","doc":"Clears the console screen.\n\nThis function only works if ANSI escape codes are enabled on the shell,\nwhich means this function is by default unavailable on Windows machines.","ref":"Signo.StdLib.html#clear/1"},{"type":"function","title":"Signo.StdLib.concat/1","doc":"Concatenates two lists or two strings.\n\n    sig> (concat '(a b) '(c d))\n    (a b c d)\n    sig> (concat \"hell\" \"o\")\n    \"hello\"","ref":"Signo.StdLib.html#concat/1"},{"type":"function","title":"Signo.StdLib.cos/1","doc":"Cosine of `x` in radians.\n\n    sig> (cos (pi))\n    -1","ref":"Signo.StdLib.html#cos/1"},{"type":"function","title":"Signo.StdLib.div/1","doc":"Divides two numbers.\n\n    sig> (/ 6 2)\n    3","ref":"Signo.StdLib.html#div/1"},{"type":"function","title":"Signo.StdLib.downcase/1","doc":"Converts all characters in the given string to lowercase.\n\n    sig> (downcase \"HELLÖ\")\n    \"hellö\"","ref":"Signo.StdLib.html#downcase/1"},{"type":"function","title":"Signo.StdLib.e/1","doc":"Euler's number.\n\nFloating point approximation of mathematical constant e.\n\n    sig> (e)\n    2.71828...","ref":"Signo.StdLib.html#e/1"},{"type":"function","title":"Signo.StdLib.eq/1","doc":"Equal to operator.\n\nReturns `#true` if the two terms are equal.\n\n    sig> (== \"same\" \"same\")\n    #true\n    sig> (== 1 1.0)\n    #true\n    sig> (== \"not\" \"same\")\n    #false","ref":"Signo.StdLib.html#eq/1"},{"type":"function","title":"Signo.StdLib.first/1","doc":"Returns the first item of a list or\nthe first Unicode grapheme in a string.\n\n    sig> (first (\"hell\" \"o\"))\n    \"hell\"","ref":"Signo.StdLib.html#first/1"},{"type":"function","title":"Signo.StdLib.gt/1","doc":"Greater-than operator.\n\nReturns `#true` if `a` is greater than `b`.\n\n    sig> (> 3 3)\n    #false","ref":"Signo.StdLib.html#gt/1"},{"type":"function","title":"Signo.StdLib.gte/1","doc":"Greater-than or equal to operator.\n\nReturns `#true` if `a` is greater than or equal to `b`.\n\n    sig> (>= 3 3)\n    #true","ref":"Signo.StdLib.html#gte/1"},{"type":"function","title":"Signo.StdLib.inspect/1","doc":"Pretty prints the given argument to stdout, returns the given\nargument.\n\n    sig> (inspect if)\n     (if)\n     (if)","ref":"Signo.StdLib.html#inspect/1"},{"type":"function","title":"Signo.StdLib.join/1","doc":"Joins the given list into a string with the second argument as seperator.\n\n    sig> (join '(2 3 4) \", \")\n    \"2, 3, 4\"","ref":"Signo.StdLib.html#join/1"},{"type":"function","title":"Signo.StdLib.last/1","doc":"Returns the last item of a list or\nthe last Unicode grapheme in a string.\n\n    sig> (last '(1 2 3))\n    3\n    sig> (last ())\n    ()\n    sig> (last \"hellö\")\n    \"ö\"\n    sig> (last \"\")\n    ()","ref":"Signo.StdLib.html#last/1"},{"type":"function","title":"Signo.StdLib.length/1","doc":"Returns the amount of elements in a list of or\nthe number of Unicode graphemes in a UTF-8 string.\n\n    sig> (length '(1 2 3))\n    3\n    sig> (length \"hellö\")\n    5","ref":"Signo.StdLib.html#length/1"},{"type":"function","title":"Signo.StdLib.ln/1","doc":"Natural (base-e) logarithm of `x`.\n\n    sig> (ln 1)\n    0","ref":"Signo.StdLib.html#ln/1"},{"type":"function","title":"Signo.StdLib.log/1","doc":"Base-10 logarithm of `x`.\n\n    sig> (log 100)\n    2","ref":"Signo.StdLib.html#log/1"},{"type":"function","title":"Signo.StdLib.logn/1","doc":"Base-`n` logarithm of `x`.\n\n    sig> (logn 2 8)\n    3","ref":"Signo.StdLib.html#logn/1"},{"type":"function","title":"Signo.StdLib.lt/1","doc":"Less-than operator.\n\nReturns `#true` if `a` is less than `b`.\n\n    sig> (< 3 3)\n    #false","ref":"Signo.StdLib.html#lt/1"},{"type":"function","title":"Signo.StdLib.lte/1","doc":"Less-than or equal to operator.\n\nReturns `#true` if `a` is less than or equal to `b`.\n\n    sig> (<= 3 3)\n    #true","ref":"Signo.StdLib.html#lte/1"},{"type":"function","title":"Signo.StdLib.mult/1","doc":"Multiplies two numbers.\n\n    sig> (* 2 3)\n    6","ref":"Signo.StdLib.html#mult/1"},{"type":"function","title":"Signo.StdLib.not_eq/1","doc":"Not equal to operator.\n\nReturns `#true` if the two terms are not equal.\n\n    sig> (!= \"same\" \"same\")\n    #false\n    sig> (!= 1 1.0)\n    #false\n    sig> (!= \"not\" \"same\")\n    #true","ref":"Signo.StdLib.html#not_eq/1"},{"type":"function","title":"Signo.StdLib.nth/1","doc":"Returns the element at `index` in a list,\nor the Unicode grapheme at `index` in a string.\n\n    sig> (nth 1 '(1 2 3))\n    2\n    sig> (nth 3 '(1 2 3))\n    ()\n    sig> (nth 4 \"hellö\")\n    \"ö\"\n    sig> (nth 5 \"hellö\")\n    ()","ref":"Signo.StdLib.html#nth/1"},{"type":"function","title":"Signo.StdLib.pi/1","doc":"Ratio of the circumference of a circle to its diameter.\n\nFloating point approximation of mathematical constant π.\n\n    sig> (pi)\n    3.14159...","ref":"Signo.StdLib.html#pi/1"},{"type":"function","title":"Signo.StdLib.pop/1","doc":"Returns a new list containing the first item of the old list\nand the remainder of the old list.\n\n    sig> (pop '(\"hell\" \"o\" \"world\"))\n    (\"hell\" (\"o\" \"world\"))\n    sig> (pop ())\n    ((), ()))\n    sig> (pop \"hello\")\n    (\"h\", \"ello\"))\n    sig> (pop \"\")\n    ((), \"\"))","ref":"Signo.StdLib.html#pop/1"},{"type":"function","title":"Signo.StdLib.pow/1","doc":"Raise `x` to the power `n`, that is `xⁿ`.\n\n    sig> (^ 2 3)\n    8","ref":"Signo.StdLib.html#pow/1"},{"type":"function","title":"Signo.StdLib.print/1","doc":"Prints a string representation of the given argument\nto stdout, and returns `#ok`.\n\nOnly works for builtin types implementing `String.Chars`:\n\n  - Strings\n  - Numbers\n  - Atoms\n  - Nil","ref":"Signo.StdLib.html#print/1"},{"type":"function","title":"Example - Signo.StdLib.print/1","doc":"sig> (print 10)\n    10\n    #ok","ref":"Signo.StdLib.html#print/1-example"},{"type":"function","title":"Signo.StdLib.product/1","doc":"Returns the product of all numbers in a list.\n\nRaises `Signo.TypeError` if one of the elements of\nthe list is not a number.\n\n    sig> (product '(2 3 4))\n    24","ref":"Signo.StdLib.html#product/1"},{"type":"function","title":"Signo.StdLib.push/1","doc":"Pushes the given item onto the end of a list.\n\nLook out: this function *only accepts lists*. To\nconcatinate strings, use `concat/2`.\n\n    sig> (push 3 '(1 2))\n    (1 2 3)","ref":"Signo.StdLib.html#push/1"},{"type":"function","title":"Signo.StdLib.sin/1","doc":"Sine of `x` in radians.\n\n    sig> (sin (pi))\n    0","ref":"Signo.StdLib.html#sin/1"},{"type":"function","title":"Signo.StdLib.sqrt/1","doc":"Square root of `x`.\n\n    sig> (sqrt 4)\n    2","ref":"Signo.StdLib.html#sqrt/1"},{"type":"function","title":"Signo.StdLib.sub/1","doc":"Substracts two numbers.\n\n    sig> (- 3 2)\n    1","ref":"Signo.StdLib.html#sub/1"},{"type":"function","title":"Signo.StdLib.sum/1","doc":"Returns the sum of all numbers in a list.\n\nRaises `Signo.TypeError` if one of the elements of\nthe list is not a number.\n\n    sig> (sum '(1 2 3))\n    6","ref":"Signo.StdLib.html#sum/1"},{"type":"function","title":"Signo.StdLib.tan/1","doc":"Tangent of `x` in radians.\n\n    sig> (tan (/ pi 4))\n    1","ref":"Signo.StdLib.html#tan/1"},{"type":"function","title":"Signo.StdLib.tau/1","doc":"Ratio of the circumference of a circle to its radius.\n\nThis constant is equivalent to a full turn when described in radians.\nSame as `(* 2 (pi))`.\n\n    sig> (tau)\n    6.28318...","ref":"Signo.StdLib.html#tau/1"},{"type":"function","title":"Signo.StdLib.tie/1","doc":"Combines all passed arguments into a list.\n\nThis function differs from creating a list by quoting\nin that here all arguments get evaluated before forming\na list, where by quoting, the entire list remaings unevaluated:\n\n    sig> '(1 2 (+ 1 2))\n    (1 2 (+ 1 2))\n    sig> (tie 1 2 (+ 1 2))\n    (1 2 3)","ref":"Signo.StdLib.html#tie/1"},{"type":"function","title":"Signo.StdLib.trim/1","doc":"Returns a string where all leading and trailing Unicode whitespaces have been removed.\n\n    sig> (trim \"   signo  \")\n    'signo'","ref":"Signo.StdLib.html#trim/1"},{"type":"function","title":"Signo.StdLib.upcase/1","doc":"Converts all characters in the given string to uppercase.\n\n    sig> (upcase \"hellö\")\n    \"HELLÖ\"","ref":"Signo.StdLib.html#upcase/1"},{"type":"module","title":"Signo.AST","doc":"AST definitions for the Signo Programming Language.","ref":"Signo.AST.html"},{"type":"type","title":"Signo.AST.callable/0","doc":"Values to which arguments can be applied,\naka the head of a procedure.","ref":"Signo.AST.html#t:callable/0"},{"type":"type","title":"Signo.AST.expression/0","doc":"An expression is a building block for the AST\nthat evaluates down to an `t:value/0`.","ref":"Signo.AST.html#t:expression/0"},{"type":"macro","title":"Signo.AST.is_value/1","doc":"","ref":"Signo.AST.html#is_value/1"},{"type":"type","title":"Signo.AST.ref/0","doc":"A reference is a key by which a `t:value/0` can\nbe looked up in the `Signo.Env`.","ref":"Signo.AST.html#t:ref/0"},{"type":"type","title":"Signo.AST.t/0","doc":"","ref":"Signo.AST.html#t:t/0"},{"type":"type","title":"Signo.AST.value/0","doc":"A value is an expression that cannot be further\nsimplied by evaluating it.","ref":"Signo.AST.html#t:value/0"},{"type":"module","title":"Signo.AST.Atom","doc":"An atom value.","ref":"Signo.AST.Atom.html"},{"type":"function","title":"Signo.AST.Atom.new/1","doc":"","ref":"Signo.AST.Atom.html#new/1"},{"type":"type","title":"Signo.AST.Atom.t/0","doc":"","ref":"Signo.AST.Atom.html#t:t/0"},{"type":"module","title":"Signo.AST.Builtin","doc":"A reference to a function in the `Signo.StdLib`.","ref":"Signo.AST.Builtin.html"},{"type":"type","title":"Signo.AST.Builtin.definition/0","doc":"","ref":"Signo.AST.Builtin.html#t:definition/0"},{"type":"function","title":"Signo.AST.Builtin.new/1","doc":"","ref":"Signo.AST.Builtin.html#new/1"},{"type":"type","title":"Signo.AST.Builtin.t/0","doc":"","ref":"Signo.AST.Builtin.html#t:t/0"},{"type":"module","title":"Signo.AST.Construct","doc":"A reference to a macro in the `Signo.SpecialForms`.","ref":"Signo.AST.Construct.html"},{"type":"type","title":"Signo.AST.Construct.definition/0","doc":"","ref":"Signo.AST.Construct.html#t:definition/0"},{"type":"function","title":"Signo.AST.Construct.new/1","doc":"","ref":"Signo.AST.Construct.html#new/1"},{"type":"type","title":"Signo.AST.Construct.t/0","doc":"","ref":"Signo.AST.Construct.html#t:t/0"},{"type":"module","title":"Signo.AST.Lambda","doc":"A declaration of a function with enclosed environment.","ref":"Signo.AST.Lambda.html"},{"type":"function","title":"Signo.AST.Lambda.new/3","doc":"","ref":"Signo.AST.Lambda.html#new/3"},{"type":"type","title":"Signo.AST.Lambda.t/0","doc":"","ref":"Signo.AST.Lambda.html#t:t/0"},{"type":"module","title":"Signo.AST.List","doc":"A data structure holding a list of expressions.\n\nInternally implemented as an Elixir list, which is in turn\nimplemented a linked list.","ref":"Signo.AST.List.html"},{"type":"function","title":"Signo.AST.List.new/2","doc":"","ref":"Signo.AST.List.html#new/2"},{"type":"type","title":"Signo.AST.List.t/0","doc":"","ref":"Signo.AST.List.html#t:t/0"},{"type":"module","title":"Signo.AST.Nil","doc":"The `nil` type.","ref":"Signo.AST.Nil.html"},{"type":"function","title":"Signo.AST.Nil.new/0","doc":"","ref":"Signo.AST.Nil.html#new/0"},{"type":"type","title":"Signo.AST.Nil.t/0","doc":"","ref":"Signo.AST.Nil.html#t:t/0"},{"type":"module","title":"Signo.AST.Number","doc":"A number value.","ref":"Signo.AST.Number.html"},{"type":"function","title":"Signo.AST.Number.new/1","doc":"","ref":"Signo.AST.Number.html#new/1"},{"type":"type","title":"Signo.AST.Number.t/0","doc":"","ref":"Signo.AST.Number.html#t:t/0"},{"type":"module","title":"Signo.AST.Quoted","doc":"An expression that is passed as-is, without being evaluated first.","ref":"Signo.AST.Quoted.html"},{"type":"function","title":"Signo.AST.Quoted.new/1","doc":"","ref":"Signo.AST.Quoted.html#new/1"},{"type":"type","title":"Signo.AST.Quoted.t/0","doc":"","ref":"Signo.AST.Quoted.html#t:t/0"},{"type":"module","title":"Signo.AST.String","doc":"A string value.","ref":"Signo.AST.String.html"},{"type":"function","title":"Signo.AST.String.new/1","doc":"","ref":"Signo.AST.String.html#new/1"},{"type":"type","title":"Signo.AST.String.t/0","doc":"","ref":"Signo.AST.String.html#t:t/0"},{"type":"module","title":"Signo.AST.Symbol","doc":"A reference to variable or function in scope.","ref":"Signo.AST.Symbol.html"},{"type":"function","title":"Signo.AST.Symbol.new/2","doc":"","ref":"Signo.AST.Symbol.html#new/2"},{"type":"type","title":"Signo.AST.Symbol.t/0","doc":"","ref":"Signo.AST.Symbol.html#t:t/0"},{"type":"exception","title":"Signo.DefinitionError","doc":"Raised when assigning directly to a namespace.","ref":"Signo.DefinitionError.html"},{"type":"exception","title":"Signo.LexError","doc":"Raised when the compiler finds an unexpected character or\nlexeme while tokenizing the source code.","ref":"Signo.LexError.html"},{"type":"exception","title":"Signo.ParseError","doc":"Raised when the compiler encounters an unexpected token,\nor when a token is missing.","ref":"Signo.ParseError.html"},{"type":"exception","title":"Signo.ReferenceError","doc":"Raised when the interpreter tries to access a reference to\na non-existant or undefined variable or function.","ref":"Signo.ReferenceError.html"},{"type":"exception","title":"Signo.RuntimeError","doc":"Raised when the compiler encounters *some* sort of error\nwhile evaluating the AST.","ref":"Signo.RuntimeError.html"},{"type":"exception","title":"Signo.TypeError","doc":"Raised when the compiler encounters mismatched types\nfor a function.","ref":"Signo.TypeError.html"},{"type":"extras","title":"Introduction","doc":"# Introduction\n\nWelcome!\n\nThis guide will attempt to teach you my lil' Lisp variant, Signo. It's a Lisp-based language, interpreted in Elixir. In the future I'm planning on basic Elixir-interop and maybe even running it on the BEAM directly, by compiling to Erlang byte code.\n\nAnyway. This guide will show you how to get up-and-running.","ref":"introduction.html"},{"type":"extras","title":"Installation - Introduction","doc":"Since Signo is interpreted using Elixir, which is in turn compiled to Erlang, you need to download both Elixir and Erlang. Take a look at [their installation guides](https://elixir-lang.org/install.html) first.\n\nAfter you've set that all up, you can download Signo via `git clone`:\n\n```shell\ngit clone https://git.dupunkto.org/axcelott/signo.git\ncd signo\nmix compile\n```","ref":"introduction.html#installation"},{"type":"extras","title":"Interactive Signo - Introduction","doc":"You can open a REPL (read-evaluate-print loop) by entering `mix repl`:\n\n```\n$ mix repl\nErlang/OTP 26 [erts-14.2.1] [source] [64-bit] [smp:11:11] [ds:11:11:10] [async-threads:1] [jit]\n\nInteractive Signo v0.0.1 (Elixir/1.16.2)\nsig(1)> \"hello world!\"\n\"hello world!\"\nsig(2)> (+ 40 2)\n42\n```\n\nGo ahead and type some expressions! To exit the REPL, press `Ctrl+C` twice.","ref":"introduction.html#interactive-signo"},{"type":"extras","title":"Running programs - Introduction","doc":"Typing stuff into that REPL is all good fun, but what if you're wanting to write some *real* programs? Well, don't worry, I've got you covered!\n\nOpen up your favorite text editor and write something along these lines to a file creatively named `hello.sg`:\n\n```lisp\n(print \"hello, world!\")\n```\n\nAnd then run it like this:\n\n```shell\n$ mix execute hello.sg\nhello, world!\n```\n\nWith all that out of the way, let's look at some of Signo's basic types!","ref":"introduction.html#running-programs"},{"type":"extras","title":"Basic types","doc":"# Basic types\n\nSigno's basic types are: numbers, atoms, booleans, strings, and nil. Lists will be discussed seperately later.\n\n```lisp\nsig> 1        ; number\nsig> 2.0      ; number\nsig> #true    ; boolean\nsig> #ok      ; atom\nsig> \"signo\"  ; string\nsig> '(1 2 3) ; list\n```","ref":"basic-types.html"},{"type":"extras","title":"Numbers - Basic types","doc":"Unlike other languages, Signo does not differentiate between integers and floats, instead opting for one universal `number` type. Internally, they are saved as Elixir `t:integer/0` and `t:float/0`, which means floats are 64-bit precision.","ref":"basic-types.html#numbers"},{"type":"extras","title":"Atoms & booleans - Basic types","doc":"An atom is a constant whose value is its own name. Some other languages call these symbols or enums. They are often useful to enumerate over distinct values, such as: `#ok, #error, #not-found`. \n\nAn atom is like an integer: it is itself. `1` is not anything else than just `1`; likewise, `#apple` is nothing else than `#apple`. Often they are used to express the state of an operation, by using values such as `#ok`, `#pending`, and `#error`.\n\nIn Signo, booleans are represented as atoms too, just like in Elixir. However, unlike in Elixir, you're not allowed to skip the leading `#`.\n\n```lisp\n#true\n#false\n```\n\nAtoms are equal if their names are equal.\n\n```lisp\nsig> (== #banana #banana)\n#true\n```","ref":"basic-types.html#atoms-booleans"},{"type":"extras","title":"Strings - Basic types","doc":"Strings in Signo are delimited by **double** quotes, and they are encoded in UTF-8:\n\n```lisp\nsig> \"hellö\"\n\"hellö\"\n```\n\nStrings can be concatenated using the `Signo.StdLib.concat/1` function from the standard library:\n\n```lisp\nsig> (concat \"hell\" \"o\")\n\"hello\"\n```\n\nYou can print a string using the `Signo.StdLib.print/1` function:\n\n```lisp\nsig> (print \"hello\")\nhello\n```","ref":"basic-types.html#strings"},{"type":"extras","title":"Nil - Basic types","doc":"In Signo, nil is represented as an empty list:\n\n```lisp\nsig> ()\n()\n```\n\nNil is the only non-boolean value that is falsy. So unlike other languages, where `\"\"` and `0` are often falsy too, the only two falsy values in Signo are `#false` and `()`.","ref":"basic-types.html#nil"},{"type":"extras","title":"Lists - Basic types","doc":"Lists are the only data structure available in Signo, and are used for expressing both program code, as well as data.\n\nEvery non-empty list (`()` being our nil type), has a *head*, which is the first element in it. The *tail* consists of the rest of the elements in the list.\n\nLists can be manipulated using a variety of functions from the `Signo.StdLib` module.","ref":"basic-types.html#lists"},{"type":"extras","title":"Languages features","doc":"# Languages features","ref":"language-features.html"},{"type":"extras","title":"Comments - Languages features","doc":"Everything after `;` to the end of the line will be ignored by the compiler. This space can be used to put comments for future humans reading your program.\n\n```lisp\n; this is a comment\n(let x 10) ; in JS: let x = 10;\n```","ref":"language-features.html#comments"},{"type":"extras","title":"References - Languages features","doc":"You can reference earlier defined variables by their identifier, which we call a *reference* in Signo. Signo is very lax regarding allowed characters in identifiers, allowing all alphanumeric characters, as well as these:\n`[\"_\", \"=\", \"+\", \"-\", \"*\", \"/\", \"^\", \"%\", \"#\", \"&\", \"@\", \"!\", \"?\", \"~\", \"<\", \">\"]`.\n\nThe only condition for an identifier is that it cannot start with a digit. The same rules apply for atoms too (but atoms can, unlike identifiers, start with a digit).\n\nSee [\"Procedures\"](procedures.md) for details on how to declare variables.","ref":"language-features.html#references"},{"type":"extras","title":"Standard library - Languages features","doc":"`Signo.StdLib` contains primitive functions for working with Signo's basic types, comparable the Elixir `Elixir.Kernel`. All functions in `Signo.StdLib` are available in global scope by default, but they can be overriden if you want.","ref":"language-features.html#standard-library"},{"type":"extras","title":"Lists & procedures","doc":"# Lists & procedures\n\nEverything we've discussed up to this point has been about **atomics**. They are the core building blocks that make up Signo programs:\n\n- Literals, such as `100`, `1.0`, `#true`, `\"hello world\"` and `#ok`.\n- References to previously defined variables or functions.\n\nWhile all fun and games, you can't write a meaningful program using just literal values. But when grouped into lists, we can do some pretty cool stuff. But first(!): let's talk about expressions.","ref":"procedures.html"},{"type":"extras","title":"Expressions - Lists & procedures","doc":"Signo is an expression-based language. That means that everything in Signo can be evaluated down to some kind of *value*, being:\n\n- A literal value.\n- A callable function or macro.\n\nIn total, Signo has three types of expressions:\n\n- Values, which can't be further evaluated.\n- Symbols, which evaluate to some earlier defined value.\n- Procedures, in which the tail of a list gets applied to its head.\n\nBecause everything is an expression, everything evaluates to *something*, meaning blocks of expressions can be nested.\n\nHere's a quick example:\n\n```lisp\nsig(1)> (def add (a) (lambda (b) (+ a b)))\nsig(2)> ((add 2) 3)\n5\n```\n\nAs you can see, the subprocedure evaluates to a lambda which is then called with the argument `3`, together adding up to `5`. This expression-based approach makes Signo extremely flexible and versatile.","ref":"procedures.html#expressions"},{"type":"extras","title":"Procedure - Lists & procedures","doc":"Whenever Signo encounters a list, the head of the list is assumed to be a callable value, and the tail of the list is assumed to be a list of arguments. The arguments get applied to the callable, which returns a value. \n\nWe call this a procedure. In Signo, there's three different types of callables:\n\n- Builtin functions, from `Signo.StdLib`.\n- Macros, which form language constructs, defined in `Signo.SpecialForms`.\n- User-defined functions.","ref":"procedures.html#procedure"},{"type":"extras","title":"Quoting & meta-programming - Lists & procedures","doc":"A perceptive reader might now wonder: how does one introduce an actual list in ones program, if lists are always treated as function calls? \n\nGreat question! Most Lisp dialects solve this by using the quoting mechanism. Simply put, the quote prevents a list--or any expression for that matter--from being evaluated. Instead, the node is passed as-is, effectively turning program code into data:\n\n```lisp\nsig> (let x '(something 10))\n(something 10)\n```\n\n> #### Some more playing around {: .neutral}\n> The variable `x` is now assigned to a list containing the symbol `print` and the number `10`. This list can be passed around, just like you would pass around any other value:\n>\n> ```lisp\n> sig> (def hello (thing) (inspect thing))\n>  (thing -> ...)\n> sig> (hello x) ; prints the value, and then returns it as well\n> (something 10)\n> ```\n>\n> It can be turned back into program code by evaluating it using `Signo.SpecialForms._eval/> 2`. But since `something` is not defined within current scope, that will raise a > ReferenceError:\n>\n> ```lisp\n> sig> (eval x)\n> [ReferenceError] 'something' is undefined at nofile:1:7\n> ```\n>\n> (However, if we define `something` beforehand, or pass some other Standard Library function, such as `print`, this would work :))\n\nThis may alert the observant reader once again; what if I want to create a list out of a set of expressions? Because if the quote stops the expression from evaluating, any nested expressions will stay untouched too. And that's correct:\n\n```lisp\nsig> '(1 2 (+ 1 2))\n(1 2 (+ 1 2))\n```\n\nThat's what the `Signo.StdLib.tie/1` function is for. The difference here is that `tie` evaluates the arguments before turning them into a list, meaning `(+ 1 2)` will be resolved before being put into the list:\n\n```lisp\nsig> (tie 1 2 (+ 1 2))\n(1 2 3)\n```","ref":"procedures.html#quoting-meta-programming"},{"type":"extras","title":"Immutability","doc":"# Immutability\n\nLastly, Signo is entirely immutable. While a variable can be reassigned within\nscope, a reference to a variable can never be mutated and then used elsewhere.\n\n```lisp\n(let x 10)\n(some-long-running-thing x)\n(let x 20) ; x reassigned here, but x in some-long-running-thing is still 10\n(print x) ; likewise, changing x in some-long-running-thing doesn't change it's value here\n```","ref":"immutability.html"},{"type":"extras","title":"Namespaces","doc":"# Namespaces\n\nWhile single-file programs are fine for small utilities and scripts, bigger projects often require splitting code into standlone modules, and sharing code between those modules.\n\nFor this purpose, Signo has two builtin language constructs:\n\n- `include` reads an external file and evaluates it, mutating global scope. Think of it as dropping the external code into the current file, ala C.\n\n- `import` works similarly, but rather than exposing and mutating global scope, the file gets evaluated in an isolated environment and the resulting scope gets merged into global scope under a namespace.\n\nThe namespace acts as a prefix for all identifiers in the resulting `Signo.Env`. Namespaced references contain a `:` character, separating the namespace from the original identifier.\n\n```signo\n# coords.sg\n(let x 10)\n(let y 20)\n\n# main.sg\n(import \"coords.sg\" as coords)\n(+ coords:x coords:y) # => 30\n```\n\nIf you omit the `as`-clause, the namespace will be inferred from the filename. If the namespace already exists, it will be masked, similar to how variables are usually masked. Also, the `as`-clause accepts both symbols, as well as strings.\n\nWhen importing a file, namespaced references from that file will be omitted:\n\n```signo\n# utils.sg\n(def hello () (print \"hello!\"))\n\n# coords.sg\n(import \"utils.sg\")\n(let x 10)\n(let y 20)\n\n# main.sg\n(import \"coords.sg\" as coords)\n(+ coords:x coords:y) # => 30\n(coords:utils:hello) # => [ReferenceError] 'coords:utils:hello' is undefined\n```\n\nIt is not possible to directly assign to a namespace either:\n\n```signo\n(let coords:x 10) # => [DefinitionError] 'coords:x' is an illegal reference\n```","ref":"namespaces.html"},{"type":"extras","title":"Appendix: Example program","doc":"# Appendix: Example program\n\n```lisp\n; this is a comment\n\n; variable definition and expressionism\n(print (let meaningOfTheUniverse 42))\n(print (* 2 meaningOfTheUniverse))\n\n; operators\n(let coolNumber 69.420)\n(print (and coolNumber #true))\n\n; simple function definitions\n(def double (n) (* 2 n))\n(let result (double 8))\n\n; if-expressions\n(if (== result 16) (print \"it works!\") (print \"it doesnt!\"))\n\n; the environment is 'enclosed' in a function upon definition\n; if it get's changed later, the function will retain values\n; as they were on definition. aka calling (closure 6) after\n; changing 'result' should not change behaviour.\n(def closure (n) (+ n result))\n(let result 2)\n(print (if (== 18 (closure 2)) \"omg closures!\"))\n\n; () is our nil type.\n; this does nothing\n(if (not #false) () (print \"oops!\"))\n\n; this is a convoluted way to print; but it works!\n((lambda (n) (print n)) (join '(\"hello\" \"world!\") \", \"))\n\n; lists :)\n(let x '(1 2))\n(let x (push 3 x))\n(print (sum x))\n\n; recursion\n; tail recursion not implemented...\n(def fact (n) (if (!= n 1) (* n (fact (- n 1))) 1))\n(print (fact 4))\n\n; inspect pretty prints any expression and returns it,\n; print stringifies compatible expressions (strings, atoms, nil, numbers)\n; and returns ok\n(inspect print)\n(print (inspect #apple)) ; prints \"#apple\" and then \"apple\"\n\n; this is a block: the last expression will be returned, and \n; it has a seperate scope, but inherits from global scope\n(print (do\n  (print \"hello from scope!\")\n  (let y (* 2 coolNumber))))\n\n(inspect 1.0)\n(inspect #apple)\n(inspect #true)\n(inspect \"blah\")\n(inspect ())\n(inspect '(wibble wobble))\n(inspect (tie \"hello\" \"world\"))\n(inspect fact)\n(inspect print)\n\n; y is undefined here because it's only defined in the block scope\n(print \"gonna crash now...\")\n(print y)\n```","ref":"example.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}